atom
  version,
  xGetFileVersionInfoSizeA, xGetFileVersionInfoA, xVerQueryValueA

if platform() = WIN32 then
  version = open_dll("version.dll")
  if version = NULL then error_message("library version.dll not found", 1) end if

  xGetFileVersionInfoSizeA = define_c_func(version, "GetFileVersionInfoSizeA",
                               { C_POINTER, C_POINTER }, C_INT)
  xGetFileVersionInfoA     = define_c_func(version, "GetFileVersionInfoA",
                               { C_POINTER, C_INT, C_INT, C_POINTER }, C_INT)
  xVerQueryValueA          = define_c_func(version, "VerQueryValueA",
                               { C_POINTER, C_POINTER, C_POINTER, C_POINTER },
                               C_INT)
end if

------------------------------------------------------------------------------

function getVerInfoSize(atom lpstrFilename)
  atom lpdwHandle -- Dummy handle parameter: DWORD
  integer res
  
  if platform() = WIN32 then
    lpdwHandle = allocate(4)
    res = c_func(xGetFileVersionInfoSizeA, {lpstrFilename, lpdwHandle})
    free(lpdwHandle)
    return res
  else
    return 0
  end if
end function

------------------------------------------------------------------------------

function getVerInfo(atom lpstrFilename, integer dwLen, atom lpData)
  if platform() = WIN32 then
    if not c_func(xGetFileVersionInfoA, {lpstrFilename, NULL, dwLen, lpData}) then
      error_message("Can't load version information", 0)
    end if
    return lpData
  else
    return 0
  end if
end function

--------------------------------------------------------------------------------

-- from win32lib tk_maths.e
function lo_word( atom pData)
  return and_bits(pData, #FFFF)
end function

--------------------------------------------------------------------------------

-- from win32lib tk_maths.e
function hi_word( atom pData)
  return and_bits(and_bits(pData, #FFFF0000) / #10000, #FFFF)
end function

------------------------------------------------------------------------------

function getVerInfoStr(atom lpData, sequence Path)
  atom pData     -- pointer to info data
  atom lDataLen  -- length of info pointed at by pData
  atom index
  sequence s

  if platform() = WIN32 then
    pData = allocate(4)
    lDataLen = allocate(4)
    s = {}
    -- Read the string: return "" if string doesn't exist
    if c_func(xVerQueryValueA, {lpData, allocate_string(Path), pData, lDataLen}) then
      index = peek4u(pData)
      s = append(s, hi_word(peek4u(index+8)))
      s = append(s, lo_word(peek4u(index+8)))
      s = append(s, hi_word(peek4u(index+12)))
      s = append(s, lo_word(peek4u(index+12)))
      return sprintf("%d.%d.%d.%d", s)
    end if
    free(pData)
    free(lDataLen)
    return ""
  else
    return ""
  end if
end function

------------------------------------------------------------------------------

global function getWinVersion(sequence FileName)
  integer dwLen     -- size of ver info buffer
  sequence res
  atom fVerInfo, lpstrFilename

  if platform() = WIN32 then
    if compare(lower(FileName[$-6..$]), "exw.exe") = 0 then
      return "3.1.1"
    else
      lpstrFilename = allocate_string(FileName)
      res = {}
      -- Get size of buffer: no ver info if size = 0
      dwLen = getVerInfoSize(lpstrFilename)
      if dwLen > 0 then
        -- Read ver info into buffer
        fVerInfo = allocate(dwLen)
        fVerInfo = getVerInfo(lpstrFilename, dwLen, fVerInfo)
        res = getVerInfoStr(fVerInfo, "\\")
        -- Free ver info buffer
        free(fVerInfo)
        free(lpstrFilename)
      else
        return "none"
      end if
    end if
    return res
  else
    return ""
  end if
end function

if platform() = WIN32 then
  arch = getenv("PROCESSOR_ARCHITECTURE")
end if

